import argparse
import io
import re
import tokenize

from pegen.tokenizer import Tokenizer
from pyasdl.parser import GeneratedParser as _ASDLParser

# Since the pegen.tokenizer.Tokenizer uses .type instead of .exact_type
# it is not trivial to change the default comment behavior. A workaround
# way is sanitizing the input before passing it into the real parser

COMMENT_PATTERN = re.compile(r"--.*?\n")


def parse(source, *, filename="<pyasdl>"):
    source = re.sub(COMMENT_PATTERN, str(), source)

    buffer_ = io.StringIO(source)
    tokengen = tokenize.generate_tokens(buffer_.readline)
    tokenizer = Tokenizer(tokengen)
    parser = _ASDLParser(tokenizer)
    tree = parser.start()

    if tree is None:
        raise parser.make_syntax_error(filename)
    return tree


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("file")
    options = parser.parse_args()

    with open(options.file) as f:
        content = f.read()

    print(parse(content))


if __name__ == "__main__":
    main()



from __future__ import annotations

import re
import token as _token
import tokenize as _tokenize
from dataclasses import dataclass, field
from enum import Enum, auto


class AST:
    pass


class Node(AST):
    pass


class Leaf(AST):
    pass


@dataclass
class Module(Node):
    name: str
    body: List[Type]


@dataclass
class Type(Node):
    name: str
    value: Node


@dataclass
class Sum(Node):
    types: List[Constructor]
    attributes: List[Field] = field(default_factory=list)


@dataclass
class Product(Node):
    fields: List[Field]
    attributes: List[Field] = field(default_factory=list)


@dataclass
class Constructor(Node):
    name: str
    fields: Optional[List[Field]] = None


class FieldQualifier(Enum):
    OPTIONAL = auto()
    SEQUENCE = auto()

    def __repr__(self):
        return f"FieldQualifier.{self.name}"


@dataclass
class Field(Leaf):
    kind: str
    name: str
    qualifier: Optional[FieldQualifier] = None


def add_comment_type(prefix):
    _tokenize.EXACT_TOKEN_TYPES[prefix] = _token.COMMENT
    _tokenize.PseudoToken = _tokenize.Whitespace + _tokenize.group(
        re.escape(prefix),
        _tokenize.PseudoExtras,
        _tokenize.Number,
        _tokenize.Funny,
        _tokenize.ContStr,
        _tokenize.Name,
    )
